# -*- encoding: cp1251 -*-
import os.path
import glob
import string
import re
import Image
import GifImagePlugin, JpegImagePlugin, PngImagePlugin
import urllib


def format_size(size):
	if size > 1024*1024:
		res = str(int((size + 0.0)/(1024*1024) * 100)/100.0) + ' M'
	elif size > 1024:
		res = str(int((size + 0.0)/1024 *100)/100.0) + ' K'
	else:
		res = str(size)
	return res
		

class C_deps_finder:
	link__rgxp = re.compile("<a .*?href *=['\\\"]?(?P<link>[^'\\\" >]+)['\\\"]?", re.S + re.I)
	imgs__rgxp = re.compile("<img .*?src *=['\\\"]?(?P<src>[^'\\\" >]+)['\\\"]?", re.S + re.I)
	def get_html_depends(self, filename):
		#print filename
		f = file(filename, 'r')
		html = string.join(f.readlines())
		f.close()
		
		link__mo = self.link__rgxp.findall(html)
		src__mo = self.imgs__rgxp.findall(html)
		links = list()
		for link in link__mo:
			pos = string.find(link, '#')
			if pos >= 0:
				link = link[:pos]
			if len(link) > 0: 
				links.append(link)
		
		images = list()
		for src in src__mo:
			images.append(src)
		
		return (links, images)
	

class C_site_checker:
		
	def __init__(self, basename):
		self.req_imgs = list()
		self.req_dirs = list()
		self.req_files = list()
		self.deps_finder = C_deps_finder()
		self.total_size_limit = (1024+512)*1024
		self.check_external = 0
		
		self.basename = basename
		if basename == '.':
			last_name = os.path.basename(os.path.abspath('.'))
		else:
			last_name = os.path.basename(basename)
		#print last_name
		try:
			conf = file('site.cfg')
		except:
			self.req_imgs.append(('photo.jpg', 180, 240, 10240))
			self.req_imgs.append((last_name+'.jpg', 90, 120, 5120))		
		
			self.req_dirs.append('diss')
			self.req_dirs.append('library')
			self.req_dirs.append('links')
			self.req_dirs.append('zvit')
	
			self.req_files.append('index.html')
			self.req_files.append('index.htm')
			self.req_files.append('indexe.htm')
			self.req_files.append('indexu.htm')
			self.req_files.append('diss/index.htm')
			self.req_files.append('links/index.htm')
			self.req_files.append('library/index.htm')
			self.req_files.append('zvit/index.htm')
		else:
			lines = conf.readlines();
			conf.close()
			for i in lines:
				#if string.find(i,'#') == 0:
				#	continue
				if string.find(i,'dir ') == 0:
					self.req_dirs.append(i[4:-1])
					#print i[5:]
				if string.find(i,'file ') == 0:
					self.req_files.append(i[5:-1])
					#print i[5:]
				if string.find(i,'image ') == 0:
					s = string.split(i[6:-1],' ')
					s[0] = re.sub('%name%', last_name,s[0])
					self.req_imgs.append((s[0], int(s[1]), int(s[2]), int(s[3])))
					#print i[6:]
				if string.find(i,'total_size ') == 0:
					self.total_size_limit = int(i[11:])
	
	def check_structure(self):
		res = list()	# error strings
		for dir in self.req_dirs:
			dirname = os.path.abspath(os.path.join(self.basename, dir))
			if not os.path.isdir(dirname):
				res.append(('struct error', '   : ' + dirname))
		
		for file in self.req_files:
			filename = os.path.abspath(os.path.join(self.basename, file))
			if not os.path.isfile(filename):
				res.append(('struct error', '   : ' + filename))
			
		for file, x, y, size in self.req_imgs:
			filename = os.path.abspath(os.path.join(self.basename, file))
			if not os.path.isfile(filename):
				res.append(('struct error', '   : ' + filename))
				continue
			
			file_size = os.path.getsize(filename) 
			if file_size > size:
				res.append(('struct error', '      ('+ format_size(size) +') : ' + filename + '  ' + format_size(file_size) + ''))
			
			img = Image.open(filename)
			im_x , im_y = img.size
			img = None
			if im_x > x :
				res.append(('struct error', '  ('+ str(x) +' ) : ' + filename + '   ' + str(im_x) + ' '))
			if im_y > y :
				res.append(('struct error', '  ('+ str(y) +' ) : ' + filename + '   ' + str(im_y) + ' '))
				
		return res;

	def check_integrity(self, path = ''):
		if path == '':
			path = self.basename
		errs, total_size = self.check_int(path)
		if total_size > self.total_size_limit :
			errs.append(('struct error', '     (' + format_size(self.total_size_limit) + ') : ' + path + '  ' + format_size(total_size) + ''))
		else:
			errs.append(('struct error', '  : ' + path + '  ' + format_size(total_size) + ''))
			
		return errs
		

	def check_int(self, path):
		"checks integrity and total size. returns (list_of_errors, total_size)"

		res = list();
		total_size = 0;
		for name in glob.glob(os.path.join(path, '*')):
			#name = os.path.join(path,name)
			#print name;
			if os.path.isdir(name):
				errs, tot_size = self.check_int(name)
				errs.append(('dir info', name + '  '+ format_size(tot_size)))
				res.extend(errs)
				total_size += tot_size
			if os.path.isfile(name):
				root, ext = os.path.splitext(os.path.basename(name))
				ext = string.lower(ext)
				if (ext == '.htm') or (ext == '.html'):
					links, images = self.deps_finder.get_html_depends(name)
					for link in links:
						if string.find(link, ':') >= 0:
							if self.check_external:
								try:
									f = urllib.urlopen(link)
								except:
									res.append(('external link info', '    ' + name + '  ' + link))
								else:
									f.close()
									res.append(('external link info', '     ' + name + '  ' + link))
							else:
								res.append(('external link info', '   ' + name + '  ' + link))
						else:
							if not os.path.exists(os.path.join(path, link)):
								res.append(('link error', '   '+ name + '  '+ link))
	
					for img in images:
						if not os.path.exists(os.path.join(path,img)):
							res.append(('link error', '   : ' + img + '   ' + name))
				elif ext == '.jpg' or ext == '.gif':
					im = Image.open(name)
					x, y = im.size
					size = os.path.getsize(name)
					res.append(('image info', name + ' '+ str(x) + 'x' + str(y) + ' ' + format_size(size))) 
						
				total_size += os.path.getsize(name)

		return res, total_size;

#for name in glob.glob('*'):
	#deps_finder = C_deps_finder()

#site_chkr = C_site_checker('.')


#for err in site_chkr.check_structure():
#	print err

#for err in site_chkr.check_integrity():
#	print err
	#if os.path.isdir(name):
	#	print name + '  dir'
	#else:
	#	print deps_finder.get_html_depends(name)
		
